"""
Convert an OpenAPI spec to a Postman collection using openapi2postmanv2.
"""
import invoke
from os.path import exists
from os import remove
from loguru import logger
from api_validator.tools.swagger_utils import add_server_entry_to_swagger_file


def openapi_to_postman(openapi_file: str, postman_file: str, server: str):
    """
    Convert an OpenAPI spec to a Postman collection using openapi2postmanv2.
    """
    logger.info(f"Converting {openapi_file} to {postman_file}...")
    if exists(postman_file):
        remove(postman_file)
    add_server_entry_to_swagger_file(
        file_path=openapi_file,
        output_file=openapi_file,
        url=server,
        description="Generated by NightVision"
    )
    try:
        command = invoke.run(f"openapi2postmanv2 --spec {openapi_file} --output {postman_file}", hide=True, in_stream=False)
        if not exists(postman_file):
            logger.error(f"Failed to convert {openapi_file} to {postman_file}")
            print(f"Return code: {command.return_code}")
            print(f"stderr: {command.stderr}")
            print(f"stdout: {command.stdout}")
            raise Exception(f"Failed to convert {openapi_file} to {postman_file}")
        logger.info(f"Converted {openapi_file} to {postman_file}.")
    except invoke.exceptions.UnexpectedExit as exc:
        logger.error(f"Failed to convert {openapi_file} to {postman_file}")
        print(f"Error: {exc.result.stderr}")
